"""任务模块常量。"""

TASK_STATUSES = (
    "research",
    "test",
    "done",
)

STATUS_ALIASES = {}

NOTE_TYPES = (
    "research",
    "test",
    "bug",
    "misc",
)

TASK_TYPES = (
    "requirement",
    "defect",
    "task",
    "risk",
)

DEFAULT_PRIORITY = 3
DEFAULT_PAGE_SIZE = 10
